%--------------------------------------------------------------------------
% COMPUTATION OF (ALMOST) FEKETE AND LEBESGUE OPTIMAL SETS.
%--------------------------------------------------------------------------
%--------------------------------------------------------------------------
%% Copyright (C) 2010
%% Matteo Briani, Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
%%
%% Author:
%%          Matteo Briani <>
%%          Alvise Sommariva  <alvise@math.unipd.it>
%%          Marco Vianello    <marcov@math.unipd.it>
%%
%% Date: December 30, 2010.
%--------------------------------------------------------------------------

clear all;

warning off;

degv=11:20;    % deg: DEGREES TO BE ANALYZED. EXAMPLE: "degv=7:12;".

family_type=2; % 1: FEKETE SET. 2: LEBESGUE SET.

% method_type_vett CHOICES:
% 1: INTERIOR POINT. 2: ACTIVE SET. 3: TRUST REGION (it needs gradients!).
% 4: fminsearch. 5: BFGS. 6:DFP 7: STEEPEST DESCENT.
method_type_vett=[4]; % EXAMPLE: method_type_vett=[2 4];

% MILDNESS PARAMETER SETS THE option PARAMETERS OF THE MINIMIZER AND
% LEBESGUE TEST SET AS WELL (FROM 0 TO 8, THE HIGHER, THE MORE TIME
% CONSUMING BUT ALSO MORE RELIABLE).
mildness_vector=[7 8]; % mildness_vector=[5 6];

starting_point=1;

% FINAL PLOTS: 0: NO 1: YES.
do_plots=0;

%--------------------------------------------------------------------------
%               ...... HERE THE MAIN PROGRAM BEGINS ......
%--------------------------------------------------------------------------

% PUT THE NAME OF YOUR FUNCTION HERE (WITHOUT .m EXTENSION).
switch family_type
    case 1
        s='set_square_fek';
    case 2
        s='set_square_leb';
end
diaryname=cat(2,'diary_',s,'.m');

diary on; diary(diaryname);

fprintf('\n \t ');
uu=cat(2,'function pts=',s,'(deg)');
fprintf(uu);

lebv=[]; detv=[]; condv=[];

fprintf('\n \n \t switch deg \n');

for kk=1:length(degv)

    deg=degv(kk);

    %--------------
    % INITIAL SET.
    %--------------

    % OPTIMIZATION STARTS FROM A "GOOD" SET. IN OUR CASE THEY ARE
    % PRECOMPUTED OR A KNOWN SET IN THE DOMAIN IN ANALYTICAL FORM.
    switch starting_point
        case 0
            if family_type == 1
                pts_start=set_square_fek(deg);
            else
                pts_start=set_square_leb(deg);
            end
        case 1
            pts_start=set_optpd_leb(deg);
    end

    %---------------
    % OPTIMIZATION.
    %---------------

    % INITIAL POINT SET AND ITS STATISTICS TO BE IMPROVED.
    pts_in=pts_start;
    [lebs,dets,conds]=do_stats(deg,pts_in,[],[],0);

    cputiming1=cputime;

    if family_type == 1
        value_new=abs(dets); value_old=abs(dets)/2; value_init=value_new;
    else
        value_new=abs(lebs); value_old=2*abs(lebs); value_init=value_new;
    end

    % OPTIMIZATION PROCESS.
    Nits=200;
    for jj=1:Nits

        if abs(value_new - value_old) > 10^(-4)*abs(value_old)

            value_old=abs(value_new);
            [pts_out,value_new]=iterative_step(deg,pts_in,...
                family_type,value_new,mildness_vector,method_type_vett);
            value_new=abs(value_new);
            pts_in=pts_out;
            jj_last=jj;

        end

    end

    if jj_last < Nits
        fprintf('\n \n \n \t %% [STABLE CONFIGURATION]: %3.0f',jj_last);
    end

    if family_type == 1
        if abs(value_new) > abs(value_init)
            fprintf('\n \t %% [+]: OLD: %2.5e NEW: %2.5e',abs(value_init),...
                abs(value_new));
        else
            fprintf('\n \n \n \t %% [=]: OLD: %2.5e NEW: %2.5e',abs(value_init),...
                abs(value_new));
        end
    else
        if abs(value_new) < abs(value_init)
            fprintf('\n \n \n \t %% [+]: OLD: %2.5e NEW: %2.5e',abs(value_init),...
                abs(value_new));
        else
            fprintf('\n \n \n \t %% [=]: OLD: %2.5e NEW: %2.5e',abs(value_init),...
                abs(value_new));
        end
    end


    cputiming2=cputime;

    %-------
    % STATS.
    %-------
    timing=cputiming2-cputiming1;
    [lebs,dets,conds]=do_stats(deg,pts_out,timing,[],1);

    lebv=[lebv lebs]; detv=[detv dets]; condv=[condv conds];

    %-------
    % PLOTS.
    %-------
    if do_plots == 1
        pts_bnd=[-1 -1; 1 -1; 1 1; -1 1; -1 -1];
        plot(pts_bnd(:,1),pts_bnd(:,2),'b-',...
            pts_out(:,1),pts_out(:,2),'r+');
    end

end

fprintf('\n \n \t otherwise');
fprintf('\n \n \t pts=pdpts(deg);');
fprintf('\n \n \t end');

fprintf('\n \t %% STATS');
fprintf('\n \t %% |DEGR|  LEBESGUE   | ABS.DET.VAND.| COND. VAND. |')
fprintf('\n \t stats_matrix=[');
for ii=1:length(degv)
    fprintf('\n \t %2.0f %1.5e %1.5e %1.5e',degv(ii),lebv(ii),...
        abs(detv(ii)),condv(ii));
end
fprintf('\n \t ]; \n');


%-----------------------
% FINAL SUMMARY.
%-----------------------

fprintf('\n \t %% ------------------------------------------------- ');
fprintf('\n \t %% |DEGR|  LEBESGUE   | ABS.DET.VAND.| COND. VAND. |')
fprintf('\n \t %% ------------------------------------------------- ');

for ii=1:length(degv)
    fprintf('\n \t %% | %2.0f | %1.5e | %1.5e | %1.5e |',degv(ii),lebv(ii),...
        abs(detv(ii)),condv(ii));
end
fprintf('\n \t %% ------------------------------------------------- \n \n');
diary off;
