
function [pts_out,value]=general_constrained_optimization_square(deg,...
    pts_in,family_type,mildness,verbosity)


%--------------------------------------------------------------------------
% INPUTS.
%---------
% deg: DEGREE OF THE FAMILY OF POINTS.
%
% pts_in: SIMPLEX POINTS. THEY ARE STORED AS AN "N x 2 MATRIX.
%
% family_type: 1: VANDERMONDE. 2: LEBESGUE CONST. OTHER: RIESZ POINTS.
%
% mildness: IT DECIDES THE MINIMIZER options. THE HIGHER THE STRONGER.
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
% pts_out: SQUARE POINTS AFTER THE OPTIMIZATION PROCESS. THEY ARE STORED
%   AS AN "N x 2 MATRIX.
%--------------------------------------------------------------------------
%%
%% Copyright (C) 2010. Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Matteo Briani
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: December 10, 2010
%%
%--------------------------------------------------------------------------

warning off;

% SETTING DEFAULTS.
leb_default=250;

% DATA BEFORE STARTING THE PROCESS.
dim_in=size(pts_in,1);
x_in=[pts_in(:,1); pts_in(:,2)];


% OPTIONS.
[options,degree_leb,maxfuns,maxtol,tolx]=option_settings_constrained(...
    deg,mildness,verbosity);

switch family_type
    case 2
        [pts_leb,w_sq]=tens_cub_sq(degree_leb,0,0,1);
        Vleb = chebvand2d(deg,pts_leb);
    otherwise
        Vleb=[];
end

%----------------------------------
% OPTIMIZATION PROCESS.
%----------------------------------

value_in=opt_target(x_in,deg,family_type,Vleb,1);

if family_type ~= 1
    ratio=1;
else
    ratio=abs(1/value_in);
end

A=[]; B=[]; Aeq=[]; Beq=[]; noncolon=[];
LB=-ones(size(x_in)); UB=ones(size(x_in));
time1=cputime;
x_out = fmincon(@(x) opt_target(x,deg,family_type,Vleb,ratio),...
    x_in,A,B,Aeq,Beq,LB,UB,noncolon,options);
time2=cputime;

value_out=opt_target(x_out,deg,family_type,Vleb,1);

% fprintf('\n \t [VALUE] IN: %2.5e OUT: %2.5e',value_in,value_out);

%----------------------------------
% SEE IF THERE ARE IMPROVEMENTS.
%----------------------------------

switch family_type

    case 1
        if abs(value_in) >= abs(value_out)
            % fprintf('\n \t %% [%1.0f][%1.3e][=] VALUE IN: %2.5e OUT: %2.5e',...
            %    mildness,time2-time1,value_in,value_out);
            x_out=x_in;
            value=value_in;
        else
            % fprintf('\n \t %% [%1.0f][%1.3e][+] VALUE IN: %2.5e OUT: %2.5e',...
            %    mildness,time2-time1,value_in,value_out);
            value=value_out;
        end

    case 2

        [pts_leb,w_sq]=tens_cub_sq(leb_default,0,0,1);
        V_leb = chebvand2d(deg,pts_leb);

        V_pts_in=chebvand2d(deg,pts_in);
        value_in=norm(V_pts_in'\V_leb',1);

        pts_out=[x_out(1:dim_in,1) x_out(dim_in+1:end,1)];
        V_pts_out = chebvand2d(deg,pts_out);
        value_out=norm(V_pts_out'\V_leb',1);

        % fprintf('\n \t %% [%1.0f][%1.3e] VALUE IN: %2.5e OUT: %2.5e',...
        %    mildness,time2-time1,value_in,value_out)

        if abs(value_in) < abs(value_out)
            x_out=x_in; value=value_in;
        else
            value=value_out;
        end

    otherwise
        if abs(value_in) < abs(value_out)
            x_out=x_in;
            value=value_in;
        else
            value=value_out;
        end

end

%----------------------------------
% REASSEMBLING POINTS.
%----------------------------------
pts_out=[x_out(1:dim_in,1) x_out(dim_in+1:end,1)];






















%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES.
%--------------------------------------------------------------------------

function [options,lebdeg,maxfuns,maxtol,tolx]=option_settings_constrained(...
    deg,mildness,verbosity)


switch mildness
    case 0
        if deg < 7
            lebdeg=70;
        else
            if deg < 12
                lebdeg=50;
            else
                lebdeg=30;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 1

        if deg < 7
            lebdeg=80;
        else
            if deg < 12
                lebdeg=60;
            else
                lebdeg=40;
            end
        end

        maxfuns=50; maxtol=10^(-12);  tolx=maxtol;
    case 2
        if deg < 7
            lebdeg=100;
        else
            if deg < 12
                lebdeg=70;
            else
                lebdeg=50;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 3
        if deg < 7
            lebdeg=135;
        else
            if deg < 12
                lebdeg=85;
            else
                lebdeg=60;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 4
        if deg < 7
            lebdeg=170;
        else
            if deg < 12
                lebdeg=120;
            else
                lebdeg=80;
            end
        end
        maxfuns=50; maxtol=10^(-14); tolx=maxtol;

    case 5

        if deg < 7
            lebdeg=250;
        else
            if deg < 12
                lebdeg=150;
            else
                lebdeg=100;
            end
        end
        maxfuns=30; maxtol=10^(-400);  tolx=10^(-60);

    case 6
        if deg < 7
            lebdeg=270;
        else
            if deg < 12
                lebdeg=170;
            else
                lebdeg=120;
            end
        end
        maxfuns=50; maxtol=10^(-400); tolx=maxtol;

    case 7

        if deg < 7
            lebdeg=300;
        else
            if deg < 12
                lebdeg=200;
            else
                lebdeg=140;
            end
        end
        maxfuns=50; maxtol=10^(-400);  tolx=10^(-60);

    case 8

        if deg < 7
            lebdeg=325;
        else
            if deg < 12
                lebdeg=250;
            else
                lebdeg=170;
            end
        end
        maxfuns=15; maxtol=10^(-400);  tolx=10^(-60);

    otherwise

        if deg < 7
            lebdeg=350;
        else
            if deg < 12
                lebdeg=250;
            else
                lebdeg=170;
            end
        end
        maxfuns=50; maxtol=10^(-400);  tolx=10^(-60);

end

switch verbosity

    case -1 % TRUST REGION.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','off',...
            'Algorithm','trust-region-reflective','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));
        
    case -2 % ACTIVE SET.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','off',...
            'Algorithm','active-set','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3)); 
        
    case -3 % INTERIOR-POINT.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','off',...
            'Algorithm','interior-point','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));
        
    case 1 % TRUST REGION.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','iter',...
            'Algorithm','trust-region-reflective','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));
        
    case 2 % ACTIVE SET.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','iter',...
            'Algorithm','active-set','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3)); 
        
           case 3 % INTERIOR-POINT.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','iter',...
            'Algorithm','interior-point','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));

end



function value=opt_target(x,deg,family_type,Vleb,ratio)

%--------------------------------------------------------------------------
% INPUTS.
%---------
% x: SQUARE POINTS ARE STORED IN THE VECTOR x. IF THEY ARE "[X Y]"
%   (N x 2 MATRIX) THEN "x=[X; Y]" (2*N x 1 MATRIX).
%
% deg: DEGREE OF THE FAMILY OF POINTS.
%
% family_type: 1: VANDERMONDE. 2: LEBESGUE CONST.
%
% Vleb: VAND. MATRIX ON SETS THE DEGREE OF A FAMILY OF POINTS OVER WHICH
%    THE LEBESGUE CONSTANT IS TESTED.
%
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
% value: VALUE OF THE DETERMINANT OR OF THE LEBESGUE CONSTANT (SEE family_type).
%--------------------------------------------------------------------------
% ROUTINES.
%----------
% 1. dubiner_leg (AND ITS SUBROUTINES!).
% 2. blyth_pozrikidis.
%--------------------------------------------------------------------------
%%
%% Copyright (C) 2010. Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Matteo Briani
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: December 10, 2010
%%
%--------------------------------------------------------------------------

if nargin < 5
    ratio=1;
end
%---------------------------------------------
% REASSEMBLING OPTIMIZATION VECTOR INTO N x 2
% MATRIX.
%--------------------------------------------
dim_space=length(x)/2;

X=x(1:dim_space);
Y=x(dim_space+1:end);

pts=[X Y];


%---------------------------------------------
% OPTIMIZATION TARGET FUNCTION..
%---------------------------------------------

s = (X < -1) + (X > 1) + (Y < -1) + (Y > 1);
sum_s=sum(s);
if sum_s > 0

    % fprintf('\n \t %% WARNING: ITERATION OUT OF DOMAIN.');

    switch family_type

        case -2
            value=10^30; return;

        case -1
            value=10^30; return;

        case 0
            value=10^30; return;

        case 1 % VANDERMONDE DETERMINANT: FEKETE POINTS.
            value=10^(-30); return;

        case 2 % APPROXIMATE LEBESGUE CONSTANT: APPROXIMATE LEBESGUE POINTS.
            value=10^30; return;

    end
end
V = chebvand2d(deg,pts);
switch family_type

    case -2
        s=-family_type;
        value=riesz_target(pts,s);

    case -1
        s=-family_type;
        value=riesz_target(pts,s);

    case 0
        s=family_type;
        value=riesz_target(pts,s);

    case 1 % VANDERMONDE DETERMINANT: FEKETE POINTS.
        detV=det(V);
        value=-abs(detV);
        %fprintf('\n \t %% %2.20e',value);

    case 2 % APPROXIMATE LEBESGUE CONSTANT: APPROXIMATE LEBESGUE POINTS.
        value=norm(V'\Vleb',1);
end


%---------------------------------------------
% NAN MANAGEMENT.
% IF NAN OCCURS SOME DEFAULTS ARE SET SO THAT
% THE ROUTINE KEEPS WORKING
%---------------------------------------------
inan=isnan(value); iinf=isinf(value);
if inan == 1 | iinf == 1
    % fprintf('\n \t %% [SINGULARITY]');
    switch family_type
        case 1
            value=10^(-15);
        otherwise
            value=10^50;
    end
end





function value=riesz_target(pts,s)

value=0;
fprintf('\n \t WARNING: NOT IMPLEMENTED YET.');


