
function [pts_out,t_out,value]=general_parm_optimization(deg,...
    jacs,family_type,mildness,verbosity)


%--------------------------------------------------------------------------
% INPUTS.
%---------
% deg: DEGREE OF THE FAMILY OF POINTS.
%
% jacs: JACOBI EXPONENTS RELATED TO PADUA POINTS FAMILY.
%
% family_type: 1: VANDERMONDE. 2: LEBESGUE CONST. OTHER: RIESZ POINTS.
%
% mildness: IT DECIDES THE MINIMIZER options. THE HIGHER THE STRONGER.
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
% pts_out: SQUARE POINTS AFTER THE OPTIMIZATION PROCESS. THEY ARE STORED
%   AS AN "N x 2 MATRIX.
%--------------------------------------------------------------------------
%%
%% Copyright (C) 2010. Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Matteo Briani
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: December 28, 2010
%%
%--------------------------------------------------------------------------

% SETTING DEFAULTS.
leb_default=250;

% DATA BEFORE STARTING THE PROCESS.
t_in=jacs;

% OPTIONS.
[options,degree_leb,maxfuns,maxtol,tolx]=option_settings_constrained(...
    deg,mildness,verbosity);

switch family_type
    case 2
        [pts_leb,w_sq]=tens_cub_sq(degree_leb,0,0,1);
        Vleb = chebvand2d(deg,pts_leb);
    otherwise
        Vleb=[];
end

%----------------------------------
% OPTIMIZATION PROCESS.
%----------------------------------
value_in=abs(opt_target(t_in,deg,family_type,Vleb,1));

if family_type ~= 1
    ratio=1;
else
    ratio=abs(1/value_in);
end

time1=cputime;

if (abs(verbosity) == 1) |  (abs(verbosity) == 2) | (abs(verbosity) == 3)
    method = 0;
end

if (abs(verbosity) == 4)
    method = 1;
end

if (abs(verbosity) == 5)
    method = 2;
end


switch method
    case 0
        A=[-1 0; 0 -1]; B=[1-10^(-3); 1-10^(-3)];
        t_out = fmincon(@(t) opt_target(t,deg,family_type,Vleb,ratio),...
            t_in,A,B,[],[],[],[],[],options);
    case 1
        t_out = fminsearch(@(t) opt_target(t,deg,family_type,Vleb,ratio),...
            t_in,options);
    case 2
        t_out = fminunc(@(t) opt_target(t,deg,family_type,Vleb,ratio),...
            t_in,options);
end
time2=cputime;

%----------------------------------
% SEE IF THERE ARE IMPROVEMENTS.
%----------------------------------

switch family_type

    case 1
        value_out=abs(opt_target(t_out,deg,family_type,Vleb,1));

        %fprintf('\n \t %% [%1.0f][%1.3e] VALUE IN: %2.5e OUT: %2.5e',...
         %   mildness,time2-time1,value_in,value_out)

        if abs(value_in) > abs(value_out)
            t_out=t_in;
            pts_in=padua_jacobi(deg,t_in(1),t_in(end),1);
            pts_out=pts_in;
            value=value_in;
        else
            value=value_out;
            pts_out=padua_jacobi(deg,t_out(1),t_out(end),1);
        end

    case 2

        [pts_leb,w_sq]=tens_cub_sq(leb_default,0,0,1);
        V_leb = chebvand2d(deg,pts_leb);

        pts_in=padua_jacobi(deg,t_in(1),t_in(end),1);
        pts_out=padua_jacobi(deg,t_out(1),t_out(end),1);

        V_pts_in=chebvand2d(deg,pts_in);
        value_in=norm(V_pts_in'\V_leb',1);

        V_pts_out = chebvand2d(deg,pts_out);
        value_out=norm(V_pts_out'\V_leb',1);

        %fprintf('\n \t %% [%1.0f][%1.3e] VALUE IN: %2.5e OUT: %2.5e',...
         %   mildness,time2-time1,value_in,value_out)

        if abs(value_in) < abs(value_out)
            pts_out=pts_in; t_out=t_in; value=value_in;
        else
            value=value_out;
        end

    otherwise

        if abs(value_in) < abs(value_out)

            pts_in=padua_jacobi(deg,t_in(1),t_in(end),1);
            pts_out=pts_in; t_out=t_in; value=value_in;
        else
            pts_out=padua_jacobi(deg,t_out(1),t_out(end),1);
            value=value_out;
        end

end






















%--------------------------------------------------------------------------
% ADDITIONAL ROUTINES.
%--------------------------------------------------------------------------

function [options,lebdeg,maxfuns,maxtol,tolx]=option_settings_constrained(...
    deg,mildness,verbosity)


switch mildness
    case 0
        if deg < 7
            lebdeg=70;
        else
            if deg < 12
                lebdeg=50;
            else
                lebdeg=30;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 1

        if deg < 7
            lebdeg=80;
        else
            if deg < 12
                lebdeg=60;
            else
                lebdeg=40;
            end
        end

        maxfuns=50; maxtol=10^(-12);  tolx=maxtol;
    case 2
        if deg < 7
            lebdeg=100;
        else
            if deg < 12
                lebdeg=70;
            else
                lebdeg=50;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 3
        if deg < 7
            lebdeg=135;
        else
            if deg < 12
                lebdeg=85;
            else
                lebdeg=60;
            end
        end
        maxfuns=50; maxtol=10^(-12); tolx=maxtol;
    case 4
        if deg < 7
            lebdeg=170;
        else
            if deg < 12
                lebdeg=120;
            else
                lebdeg=80;
            end
        end
        maxfuns=50; maxtol=10^(-14); tolx=maxtol;

    case 5

        if deg < 7
            lebdeg=250;
        else
            if deg < 12
                lebdeg=150;
            else
                lebdeg=100;
            end
        end
        maxfuns=30; maxtol=10^(-400);  tolx=10^(-60);

    case 6
        if deg < 7
            lebdeg=270;
        else
            if deg < 12
                lebdeg=170;
            else
                lebdeg=120;
            end
        end
        maxfuns=50; maxtol=10^(-400); tolx=maxtol;

    case 7

        if deg < 7
            lebdeg=300;
        else
            if deg < 12
                lebdeg=200;
            else
                lebdeg=140;
            end
        end
        maxfuns=50; maxtol=10^(-400);  tolx=10^(-60);

    case 8

        if deg < 7
            lebdeg=325;
        else
            if deg < 12
                lebdeg=250;
            else
                lebdeg=170;
            end
        end
        maxfuns=15; maxtol=10^(-400);  tolx=10^(-60);

    otherwise

        if deg < 7
            lebdeg=350;
        else
            if deg < 12
                lebdeg=250;
            else
                lebdeg=170;
            end
        end
        maxfuns=50; maxtol=10^(-400);  tolx=10^(-60);

end

switch verbosity

    case -1 % TRUST REGION.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','off',...
            'Algorithm','trust-region-reflective','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));

    case -2 % ACTIVE SET.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','off',...
            'Algorithm','active-set','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));

    case -3 % INTERIOR-POINT.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','off',...
            'Algorithm','interior-point','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));

    case 1 % TRUST REGION.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','iter',...
            'Algorithm','trust-region-reflective','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));

    case 2 % ACTIVE SET.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','iter',...
            'Algorithm','active-set','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));

    case 3 % INTERIOR-POINT.
        options = optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','iter',...
            'Algorithm','interior-point','TolCon',...
            maxtol,'ObjectiveLimit',1e300,'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));
        
    otherwise
        options=optimset('MaxIter',maxfuns,'MaxFunEvals',...
            maxfuns*10^5,'Display','off',...
            'TolCon',maxtol,'ObjectiveLimit',1e300,...
            'TolX',tolx,'TolFun',tolx,...
            'FunValCheck','on','RelLineSrchBnd',10^(-3),...
            'RelLineSrchBndDuration',maxfuns,'DiffMaxChange',10^(-3));

end



function value=opt_target(t,deg,family_type,Vleb,ratio)

%--------------------------------------------------------------------------
% INPUTS.
%---------
% t: JACOBI EXPONENT.
%
% deg: DEGREE OF THE FAMILY OF POINTS.
%
% family_type: 1: VANDERMONDE. 2: LEBESGUE CONST.
%
% Vleb: VAND. MATRIX ON SETS THE DEGREE OF A FAMILY OF POINTS OVER WHICH
%    THE LEBESGUE CONSTANT IS TESTED.
%
%--------------------------------------------------------------------------
% OUTPUTS.
%----------
% value: VALUE OF THE DETERMINANT OR OF THE LEBESGUE CONSTANT (SEE family_type).
%--------------------------------------------------------------------------
% ROUTINES.
%----------
% 1. dubiner_leg (AND ITS SUBROUTINES!).
% 2. blyth_pozrikidis.
%--------------------------------------------------------------------------
%%
%% Copyright (C) 2010. Alvise Sommariva, Marco Vianello.
%%
%% This program is free software; you can redistribute it and/or modify
%% it under the terms of the GNU General Public License as published by
%% the Free Software Foundation; either version 2 of the License, or
%% (at your option) any later version.
%%
%% This program is distributed in the hope that it will be useful,
%% but WITHOUT ANY WARRANTY; without even the implied warranty of
%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%% GNU General Public License for more details.
%%
%% You should have received a copy of the GNU General Public License
%% along with this program; if not, write to the Free Software
%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
%%
%% Author:  Matteo Briani
%%          Alvise Sommariva <alvise@euler.math.unipd.it>
%%          Marco Vianello   <marcov@euler.math.unipd.it>
%%
%% Date: December 10, 2010
%%
%--------------------------------------------------------------------------

if nargin < 5
    ratio=1;
end

%---------------------------------------------
% REASSEMBLING OPTIMIZATION VECTOR INTO N x 2
% MATRIX.
%--------------------------------------------

pts=padua_jacobi(deg,t(1),t(end),1);

%---------------------------------------------
% OPTIMIZATION TARGET FUNCTION..
%---------------------------------------------

s = (t(1) <= -1+10^(-3)) + (t(end) < -1+10^(-3));
sum_s=sum(s);
if sum_s > 0

    %fprintf('\n \t %% WARNING: ITERATION OUT OF DOMAIN.');

    switch family_type

        case -2
            value=10^30; return;

        case -1
            value=10^30; return;

        case 0
            value=10^30; return;

        case 1 % VANDERMONDE DETERMINANT: FEKETE POINTS.
            value=-ratio*10^(-30); return;

        case 2 % APPROXIMATE LEBESGUE CONSTANT: APPROXIMATE LEBESGUE POINTS.
            value=10^30; return;

    end
end
V = chebvand2d(deg,pts);
switch family_type

    case -2
        s=-family_type;
        value=riesz_target(pts,s);

    case -1
        s=-family_type;
        value=riesz_target(pts,s);

    case 0
        s=family_type;
        value=riesz_target(pts,s);

    case 1 % VANDERMONDE DETERMINANT: FEKETE POINTS.
        detV=det(V);
        value=-ratio*abs(detV);
        %fprintf('\n \t %% %2.20e',value);

    case 2 % APPROXIMATE LEBESGUE CONSTANT: APPROXIMATE LEBESGUE POINTS.
        value=norm(V'\Vleb',1);
end


%---------------------------------------------
% NAN MANAGEMENT.
% IF NAN OCCURS SOME DEFAULTS ARE SET SO THAT
% THE ROUTINE KEEPS WORKING
%---------------------------------------------
inan=isnan(value); iinf=isinf(value);
if inan == 1 | iinf == 1
    % fprintf('\n \t %% [SINGULARITY]');
    switch family_type
        case 1
            value=-ratio*10^(-30);
        otherwise
            value=10^50;
    end
end




function value=riesz_target(pts,s)

value=0;
fprintf('\n \t WARNING: NOT IMPLEMENTED YET.');





function pts=padua_jacobi(N,a,b,gl)

if (gl == 1)
    M=N-1;
else
    M=N+1;
end

[x1,w1]=gauss_jacobi(M,a,b,gl);
[x2,w2]=gauss_jacobi(M+1,a,b,gl);

% LENGTH OF THE VECTOR x1 IS "M".
L1=length(x1);

ie1=1:2:L1;
io1=2:2:L1;

L2=length(x2);

ie2=1:2:L2;
io2=2:2:L2;

[X1,Y1]=meshgrid(x1(ie1),x2(io2));
[X2,Y2]=meshgrid(x1(io1),x2(ie2));

pts(:,1)=[X1(:); X2(:)];
pts(:,2)=[Y1(:); Y2(:)];






function [x,w]=gauss_jacobi(N,a,b,gl)

% GAUSS-JACOBI (LOBATTO) RULE ON (-1,1).
% N: FOR GAUSSIAN RULES IT IS THE NUMBER OF INTERNAL QUAD. POINTS.
%    FOR THE LOBATTO CASE THE TOTAL NUMBER OF POINTS IS "N+2".
% a,b ARE THE GAUSS-JACOBI EXPONENTS.
% gl: 0: GAUSS POINTS. 1: GAUSS-LOBATTO POINTS.
% x, w ARE COLUMN VECTORS OF NODES AND WEIGHTS.
%      THE LENGTH OF x AND w IS "N" IF gl=0, "N+2" IF "gl=1".

if nargin < 2
    a=0; b=0;
end

if nargin < 4
    gl = 0;
end

if gl == 0
    ab=r_jacobi(N,a,b);
    xw=gauss(N,ab);
else
    xw=lobatto_jacobi(N,a,b);
end

x=xw(:,1);
w=xw(:,2);


%--------------------------------------------------------------------------
% ADDITIONAL FUNCTIONS BY D.LAURIE AND W.GAUTSCHI.
%--------------------------------------------------------------------------

function ab=r_jacobi(N,a,b)

nu=(b-a)/(a+b+2);
mu=2^(a+b+1)*gamma(a+1)*gamma(b+1)/gamma(a+b+2);
if N==1
    ab=[nu mu]; return
end

N=N-1;
n=1:N;
nab=2*n+a+b;
nuadd=(b^2-a^2)*ones(1,N)./(nab.*(nab+2));
A=[nu nuadd];
n=2:N;
nab=nab(n);
B1=4*(a+1)*(b+1)/((a+b+2)^2*(a+b+3));
B=4*(n+a).*(n+b).*n.*(n+a+b)./((nab.^2).*(nab+1).*(nab-1));
abadd=[mu; B1; B'];
ab=[A' abadd];

function xw=gauss(N,ab)
N0=size(ab,1); if N0<N, error('input array ab too short'), end
J=zeros(N);
for n=1:N, J(n,n)=ab(n,1); end
for n=2:N
    J(n,n-1)=sqrt(ab(n,2));
    J(n-1,n)=J(n,n-1);
end
[V,D]=eig(J);
[D,I]=sort(diag(D));
V=V(:,I);
xw=[D ab(1,2)*V(1,:)'.^2];

function xw=lobatto_jacobi(N,a,b)

if nargin<2, a=0; end;
if nargin<3, b=a; end
ab=r_jacobi(N+2,a,b);
ab(N+2,1)=(a-b)/(2*N+a+b+2);
ab(N+2,2)=4*(N+a+1)*(N+b+1)*(N+a+b+1)/((2*N+a+b+1)*(2*N+a+b+2)^2);
xw=gauss(N+2,ab);






